@@ -1,8 +1,9 @@
+# encoding: utf-8
 """
 lazyarray is a Python package that provides a lazily-evaluated numerical array
 class, ``larray``, based on and compatible with NumPy arrays.
 
-Copyright Andrew P. Davison, 2012
+Copyright Andrew P. Davison, Joël Chavas 2012-2014
 """
 from __future__ import division
 import numpy
@@ -12,7 +13,7 @@ import collections
 from functools import wraps
 import logging
 
-__version__ = "0.2.2"
+__version__ = "0.2.3"
 
 # stuff for Python 3 compatibility
 try:
@@ -72,22 +73,49 @@ def partial_shape(addr, full_shape):
     """
     Calculate the size of the sub-array represented by `addr`
     """
+    def is_boolean_array(arr):
+        return isinstance(arr, collections.Sized) and hasattr(arr, 'dtype') and arr.dtype == bool
+
     def size(x, max):
-        if isinstance(x, (int, long)):
-            return 1
+        if isinstance(x, (int, long, numpy.integer)):
+            return 1, 'int'
         elif isinstance(x, slice):
-            return 1 + ((x.stop or max) - (x.start or 0) - 1) // (x.step or 1)
+            return 1 + ((x.stop or max) - (x.start or 0) - 1) // (x.step or 1), 'slice'
         elif isinstance(x, collections.Sized):
             if hasattr(x, 'dtype') and x.dtype == bool:
-                return x.sum()
+                return (x == True).sum(), 'bool'
             else:
-                return len(x)
-    addr = full_address(addr, full_shape)
-    shape = [size(x, max) for (x, max) in zip(addr, full_shape)]
-    if shape == [1] or shape == [1, 1]:
+                return len(x), 'int'
+        else:
+            raise TypeError("Unsupported index type %s" % type(x))
+
+    def is_there_one_zero_dim(shape):
+        return bool(shape.count(0))
+
+    def is_only_one_dims(shape):
+        return len(shape) == shape.count(1)
+
+    if is_boolean_array(addr):
+        shape = [(addr == True).sum()]
+        type_dim = ['bool']
+    else:
+        addr = full_address(addr, full_shape)
+        shape, type_dim = zip(*[size(x, max) for (x, max) in zip(addr, full_shape)])
+
+    if is_there_one_zero_dim(shape):
+        return [0]
+    elif is_only_one_dims(shape) and (type_dim.count('bool') > 0):
+        return [1]
+    elif type_dim.count('slice') > 0:
+        new_shape = []
+        for i, t in enumerate(type_dim):
+            if t == 'slice' or shape[i] > 1:
+                new_shape.append(shape[i])
+        return new_shape
+    elif is_only_one_dims(shape) and (type_dim.count('int') > 0):
         return [1]
     else:
-        return [x for x in shape if x > 1] # remove empty dimensions
+        return [x for x in shape if x > 1]  # remove empty dimensions
 
 
 def reverse(func):
@@ -165,13 +193,13 @@ class larray(object):
             self._shape = shape or value.shape
             self.base_value = value.base_value
             self.dtype = dtype or value.dtype
-            self.operations = value.operations # should deepcopy?
+            self.operations = value.operations  # should deepcopy?
         elif isinstance(value, collections.Sized):  # False for numbers, generators, functions, iterators
             #assert numpy.isreal(value).all()
             if not isinstance(value, numpy.ndarray):
                 value = numpy.array(value, dtype=dtype)
             elif dtype is not None:
-                assert value.dtype == dtype # or could convert value to the provided dtype
+                assert value.dtype == dtype  # or could convert value to the provided dtype
             if shape and value.shape != shape:
                 raise ValueError("Array has shape %s, value has shape %s" % (shape, value.shape))
             self._shape = value.shape
@@ -206,7 +234,15 @@ class larray(object):
                 obj.operations.append((deepcopy(f), deepcopy(arg)))
         return obj
 
+    def __repr__(self):
+        return "<larray: base_value=%r shape=%r dtype=%r, operations=%r>" % (self.base_value,
+                                                                             self.shape,
+                                                                             self.dtype,
+                                                                             self.operations)
+
     def _set_shape(self, value):
+        if hasattr(self.base_value, "shape") and self.base_value.shape != value:
+            raise ValueError("Lazy array has fixed shape %s, cannot be changed to %s" % (self.base_value.shape, value))
         self._shape = value
         for op in self.operations:
             if isinstance(op[1], larray):
@@ -238,9 +274,9 @@ class larray(object):
     @property
     def is_homogeneous(self):
         """True if all the elements of the array are the same."""
-        hom_base = isinstance(self.base_value, (int, long, float, bool)) or type(self.base_value) == self.dtype
-        hom_ops = all(isinstance(obj.base_value, (int, long, float, bool))
-                      for obj in self.operations if  isinstance(obj, larray))
+        hom_base = isinstance(self.base_value, (int, long, numpy.integer, float, bool)) or type(self.base_value) == self.dtype
+        hom_ops = all(isinstance(obj.base_value, (int, long, numpy.integer, float, bool))
+                      for obj in self.operations if isinstance(obj, larray))
         return hom_base and hom_ops
 
     def _partial_shape(self, addr):
@@ -252,20 +288,18 @@ class larray(object):
     def _homogeneous_array(self, addr):
         self.check_bounds(addr)
         shape = self._partial_shape(addr)
-        if shape == [1]:
-            return 1
-        else:
-            return numpy.ones(shape, type(self.base_value))
+        return numpy.ones(shape, type(self.base_value))
 
     def _full_address(self, addr):
         return full_address(addr, self._shape)
 
     def _array_indices(self, addr):
         self.check_bounds(addr)
+
         def axis_indices(x, max):
-            if isinstance(x, (int, long)):
+            if isinstance(x, (int, long, numpy.integer)):
                 return x
-            elif isinstance(x, slice): # need to handle negative values in slice
+            elif isinstance(x, slice):  # need to handle negative values in slice
                 return numpy.arange((x.start or 0),
                                     (x.stop or max),
                                     (x.step or 1),
@@ -275,6 +309,8 @@ class larray(object):
                     return numpy.arange(max)[x]
                 else:
                     return numpy.array(x)
+            else:
+                raise TypeError("Unsupported index type %s" % type(x))
         addr = self._full_address(addr)
         indices = [axis_indices(x, max) for (x, max) in zip(addr, self._shape)]
         if len(indices) == 1:
@@ -283,7 +319,7 @@ class larray(object):
             if isinstance(indices[0], collections.Sized):
                 if isinstance(indices[1], collections.Sized):
                     mesh_xy = numpy.meshgrid(*indices)
-                    return (mesh_xy[0].T, mesh_xy[1].T) # meshgrid works on (x,y), not (i,j)
+                    return (mesh_xy[0].T, mesh_xy[1].T)  # meshgrid works on (x,y), not (i,j)
             return indices
         else:
             raise NotImplementedError("Only 1D and 2D arrays supported")
@@ -320,7 +356,7 @@ class larray(object):
                 n = reduce(operator.mul, partial_shape)
             else:
                 n = 0
-            base_val = self.base_value.next(n) # note that the array contents will depend on the order of access to elements
+            base_val = self.base_value.next(n)  # note that the array contents will depend on the order of access to elements
             if n == 1:
                 base_val = base_val[0]
             elif partial_shape and base_val.shape != partial_shape:
@@ -336,30 +372,46 @@ class larray(object):
         """
         Check whether the given address is within the array bounds.
         """
+        def is_boolean_array(arr):
+            return hasattr(arr, 'dtype') and arr.dtype == bool
+
         def check_axis(x, size):
-            if isinstance(x, (int, long)):
+            if isinstance(x, (int, long, numpy.integer)):
                 lower = upper = x
             elif isinstance(x, slice):
                 lower = x.start or 0
-                upper = x.stop or size-1
+                upper = x.stop or size - 1
             elif isinstance(x, collections.Sized):
-                if len(x) == 0:
-                    raise ValueError("Empty address component (address was %s)" % str(addr))
-                if hasattr(x, "min"):
-                    lower = x.min()
+                if is_boolean_array(x):
+                    lower = 0
+                    upper = x.size - 1
                 else:
-                    lower = min(x)
-                if hasattr(x, "max"):
-                    upper = x.max()
-                else:
-                    upper = max(x)
+                    if len(x) == 0:
+                        raise ValueError("Empty address component (address was %s)" % str(addr))
+                    if hasattr(x, "min"):
+                        lower = x.min()
+                    else:
+                        lower = min(x)
+                    if hasattr(x, "max"):
+                        upper = x.max()
+                    else:
+                        upper = max(x)
             else:
-                raise TypeError("Invalid array address: %s" % addr)
+                raise TypeError("Invalid array address: %s (element of type %s)" % (str(addr), type(x)))
             if (lower < -size) or (upper >= size):
                 raise IndexError("Index out of bounds")
         full_addr = self._full_address(addr)
-        for i, size in zip(full_addr, self._shape):
-            check_axis(i, size)
+        if is_boolean_array(full_addr[0]):
+            if len(addr.shape) > len(self._shape):
+                raise IndexError("Too many indices for array")
+            for xmax, size in zip(addr.shape, self._shape):
+                lower = 0
+                upper = xmax - 1
+                if (lower < 0) or (upper >= size):
+                    raise IndexError("index %s out of bounds for axis %s of size %s" % (x.size, i, size))
+        else:
+            for i, size in zip(full_addr, self._shape):
+                check_axis(i, size)
 
     def apply(self, f):
         """
@@ -431,33 +483,32 @@ class larray(object):
         else:
             raise Exception("larray is not callable")
 
-
     __iadd__ = lazy_inplace_operation('add')
     __isub__ = lazy_inplace_operation('sub')
     __imul__ = lazy_inplace_operation('mul')
     __idiv__ = lazy_inplace_operation('div')
     __ipow__ = lazy_inplace_operation('pow')
 
-    __add__  = lazy_operation('add')
+    __add__ = lazy_operation('add')
     __radd__ = __add__
-    __sub__  = lazy_operation('sub')
+    __sub__ = lazy_operation('sub')
     __rsub__ = lazy_operation('sub', reversed=True)
-    __mul__  = lazy_operation('mul')
+    __mul__ = lazy_operation('mul')
     __rmul__ = __mul__
-    __div__  = lazy_operation('div')
+    __div__ = lazy_operation('div')
     __rdiv__ = lazy_operation('div', reversed=True)
     __truediv__ = lazy_operation('truediv')
     __rtruediv__ = lazy_operation('truediv', reversed=True)
-    __pow__  = lazy_operation('pow')
+    __pow__ = lazy_operation('pow')
 
-    __lt__   = lazy_operation('lt')
-    __gt__   = lazy_operation('gt')
-    __le__   = lazy_operation('le')
-    __ge__   = lazy_operation('ge')
+    __lt__ = lazy_operation('lt')
+    __gt__ = lazy_operation('gt')
+    __le__ = lazy_operation('le')
+    __ge__ = lazy_operation('ge')
 
-    __neg__  = lazy_unary_operation('neg')
-    __pos__  = lazy_unary_operation('pos')
-    __abs__  = lazy_unary_operation('abs')
+    __neg__ = lazy_unary_operation('neg')
+    __pos__ = lazy_unary_operation('pos')
+    __abs__ = lazy_unary_operation('abs')
 
 
 def _build_ufunc(func):
